# -*- coding: utf-8 -*-
# Copyright 2023-2024 Rumma & Ko Ltd
# License: GNU Affero General Public License v3 (see file COPYING for details)
# pm test tests.test_system_tasks
"""Runs some tests on background tasks.

"""

import unittest
from django.conf import settings
from django.utils import timezone
# from mock import patch
from asgiref.sync import async_to_sync

from lino.api import dd, rt
# from lino.modlib.linod.tasks import Tasks
from lino.utils.djangotest import TestCase
from lino.modlib.linod.mixins import start_task_runner


# @unittest.skip("Not yet rewritten after 20250620")
class TestCase(TestCase):

    fixtures = ["demo_users"]

    def test_linod_tasks(self):
        # 20260516 why was this here? settings.SITE.the_demo_date = None
        SystemTask = rt.models.linod.SystemTask
        # InvoicingTask = rt.models.invoicing.Task

        ar = rt.login(dd.plugins.users.demo_username)

        with ar.capture_logger("INFO") as out:
            next_time = async_to_sync(start_task_runner)(ar, max_count=1)

        # print("20231021 ===\n" + out.getvalue() + "===")
        self.maxDiff = None
        self.assertEqual(
            out.getvalue(), """\
Start task runner using <StringLogger lino~ (INFO)>...
Update summary data for Subscriptions ...
Update summary data for Tickets ...
Update summary data for User Statistics ...
Update summary data for Order summaries ...
Update summary data for User summaries ...
Run 1 data checkers on 0 Vouchers...
Run 1 data checkers on 0 Delivery notes...
Run 1 data checkers on 0 Subscriptions...
Run 1 data checkers on 0 Trading invoices...
Run 1 data checkers on 0 Ledger invoices...
Run 3 data checkers on 0 Partners...
Run 6 data checkers on 0 Calendar entries...
Run 3 data checkers on 0 Comments...
Run 1 data checkers on 0 Places...
Run 2 data checkers on 0 Product Categories...
Run 2 data checkers on 0 Products...
Run 1 data checkers on 0 Ibanity suppliers...
Run 1 data checkers on 0 Excerpts...
Run 1 data checkers on 0 Payment terms...
Run 1 data checkers on 0 Calendars...
Run 1 data checkers on 0 Calendar entry types...
Run 1 data checkers on 0 Recurring events...
Run 1 data checkers on 0 Rooms...
Run 1 data checkers on 0 Tasks...
Run 1 data checkers on 0 Teams...
Run 2 data checkers on 0 Tickets...
Run 1 data checkers on 0 Text Field Templates...
Run 1 data checkers on 0 Interests...
Run 1 data checkers on 0 Trading invoice items...
Run 2 data checkers on 0 Working sessions...
Run 1 data checkers on 0 Upload files...
Run 2 checkers on unbound data...
- uploads.UploadsFolderChecker : (★) File uploads/orphan.txt has no upload entry.
Found 0 and fixed 1 problems in unbound data.
27 checks have been run. Found 0 and fixed 1 problems.
Send email '[noi1e] Week 20 activity report' from root@localhost to []
Ignoring email '[noi1e] Week 20 activity report' because there is no recipient
Stop after 1 loops.
""")
        # out = ar.end_log_capture()
        seconds = (next_time - timezone.now()).total_seconds()
        # seconds is quite exactly 10 seconds (the value of
        # send_pending_emails_often)
        # self.assertEqual(seconds, 20)
        self.assertTrue(seconds > 3)
        self.assertTrue(seconds < 11)
        # self.assertGreater(next_time, now())
        # _now = now()
        # for rule in SystemTask.objects.all():
        #     rule.start_job(ar)
        # latest_jobs = SystemTask.objects.all()  # filter(start_time__gt=_now)
        # self.assertEqual(latest_jobs.count(), 6)
        # print(out)
        # self.assertEqual(out, "")
        msg = '\n'.join([job.message
                         for job in SystemTask.objects.all()]).strip()
        # print("20231021 ===\n" + msg + "===")
        expected = """\
<pre></pre>

<pre>Update summary data for Subscriptions ...
Update summary data for Tickets ...
Update summary data for User Statistics ...
Update summary data for Order summaries ...
Update summary data for User summaries ...
</pre>
<pre>Run 1 data checkers on 0 Vouchers...
Run 1 data checkers on 0 Delivery notes...
Run 1 data checkers on 0 Subscriptions...
Run 1 data checkers on 0 Trading invoices...
Run 1 data checkers on 0 Ledger invoices...
Run 3 data checkers on 0 Partners...
Run 6 data checkers on 0 Calendar entries...
Run 3 data checkers on 0 Comments...
Run 1 data checkers on 0 Places...
Run 2 data checkers on 0 Product Categories...
Run 2 data checkers on 0 Products...
Run 1 data checkers on 0 Ibanity suppliers...
Run 1 data checkers on 0 Excerpts...
Run 1 data checkers on 0 Payment terms...
Run 1 data checkers on 0 Calendars...
Run 1 data checkers on 0 Calendar entry types...
Run 1 data checkers on 0 Recurring events...
Run 1 data checkers on 0 Rooms...
Run 1 data checkers on 0 Tasks...
Run 1 data checkers on 0 Teams...
Run 2 data checkers on 0 Tickets...
Run 1 data checkers on 0 Text Field Templates...
Run 1 data checkers on 0 Interests...
Run 1 data checkers on 0 Trading invoice items...
Run 2 data checkers on 0 Working sessions...
Run 1 data checkers on 0 Upload files...
Run 2 checkers on unbound data...
- uploads.UploadsFolderChecker : (★) File uploads/orphan.txt has no upload entry.
Found 0 and fixed 1 problems in unbound data.
27 checks have been run. Found 0 and fixed 1 problems.
</pre>
<pre>Send email '[noi1e] Week 20 activity report' from root@localhost to []
Ignoring email '[noi1e] Week 20 activity report' because there is no recipient
</pre>
<pre></pre>


<pre></pre>
<pre></pre>"""
        self.assertEqual(msg, expected)

        # self.assertEqual(logger.debug.call_count, 27)
        # self.assertEqual(logger.info.call_count, 13)
